import pathlib

import regression_model

import pandas as pd

pd.options.display.max_rows = 10
pd.options.display.max_columns = 10

PACKAGE_ROOT = pathlib.Path(regression_model.__file__).resolve().parent
TRAINED_MODEL_DIR = PACKAGE_ROOT / "trained_models"
DATASET_DIR = PACKAGE_ROOT / "datasets"

#data
TESTING_DATA_FILE = "test.csv"
TRAINING_DATA_FILE = "train.csv"
TARGET = "Energy_kWh"
DATETIME_INDEX = "datetime_converted"
PREDICTION_PLACE = 'Kumpula'
TEST_SIZE = 0.2
RANDOM_STATE = 42
#special vars
RADIATION_ACCUMULATION = 'radiation_global_accumulation'
GLOBAL_RADIATION = 'GlobalRadiation_W_m2'

#fmi_weather related vars
FORECAST_TO_DROP = [
    'precipitation',
    'weather_symbol',
    'radiation_global_accumulation',
    'radiation_long_wave_accumulation',
    'radiation_netsurface_long_wave_accumulation',
    'radiation_netsurface_short_wave_accumulation',
    'radiation_diffuse_accumulation'
]
#used to rename variables after merging forecasts from FMI_weather into a DF
# so that they comply with our variables names
FEATURES = [
            'datetime_converted',
            'AirTemperature_degC',
            'WindSpeed_m_s',
            'GustSpeed_m_s',
            'WindDirection',
            'RelativeHumidity_percent',
            'CloudAmount',
            'Pressure_msl_hpa',
            'DewPointTemperature_degC',
            'PrecipitationIntensity_mm_h',
            'GlobalRadiation_W_m2'
]

#variables

WIND_DISCRETE = 'WindDirection'

NUM_TO_BINARY = [
    'CloudAmount',
    'PrecipitationIntensity_mm_h'
]

BINARY_BOUNDARIES = {
    'CloudAmount': 5,
    'PrecipitationIntensity_mm_h': 0
}

TEMPORAL_HOUR = 'Hour'

TEMPORAL_DAY = 'DayofYear'

SOLAR_ANGLE = 'SolarElevationAngle_deg'

SUN_AZIMUTH = 'SunAzimuth_deg'

THEOR_SRAD = 'TheoreticalSolarRadiation'

#variables that can be later dropped after used in calculation:
DROP_FEATURES = [
    'GustSpeed_m_s',
    'datetime_converted'
]

# think of more stuff here

PIPELINE_NAME  = "rf_pipeline"
PIPELINE_SAVE_FILE = f"{PIPELINE_NAME}_output_v"

#for differential testing
ACCEPTABLE_MODEL_DIFFERENCE = 0.05



import numpy as np
import pandas as pd
from sklearn.base import BaseEstimator, TransformerMixin

from regression_model.processing.errors import InvalidModelInputError
from regression_model.processing.solar_calculation_functions import *

class WindDiscretizer(BaseEstimator, TransformerMixin):
    '''Discretization of Wind'''
    def __init__(self, variables=None):
        if not isinstance(variables, list):
            self.variables = [variables]
        else:
            self.variables = variables

    def fit(self, X, y=None):
        return self

    def transform(self, X):
        X = X.copy()
        buckets = np.arange(11.25, 372, 22.5)
        labels = np.arange(17)
        for feature in self.variables:
            X[feature].loc[(X[feature]>=0)&(X[feature]<11.25)] = X[feature].loc[(X[feature]>=0)&(X[feature]<11.25)].apply(lambda x:x+360)
            X[feature]=labels[np.digitize(X[feature], buckets)]
        return X

class DiscretizerNumericalIntoBinary(BaseEstimator, TransformerMixin):
    '''Discretization of cloud coverage and precipitation intensity into
        binary. The function takes a dictionary of variables as boundaries.
    '''

    def __init__(self, boundaries=None, variables=None):
        if not isinstance(variables, list):
            self.variables = [variables]
        else:
            self.variables = variables
        self.boundaries=boundaries

    def fit(self, X, y=None):
        return self

    def transform(self, X):
        X = X.copy()
        for feature in self.variables:
            X[feature] = np.where(X[feature]>self.boundaries[feature],1,0)
        return X

class TemporalHour(BaseEstimator, TransformerMixin):
        def __init__(self, variable=None, ref_feature=None):
            self.variable = variable
            self.ref_feature = ref_feature
            #can make one temporal transformer for both issues
            #dict with lambda?

        def fit(self, X, y=None):
            return self

        def transform(self, X):
            X = X.copy()
            X[self.variable] = pd.DatetimeIndex(pd.to_datetime(X[self.ref_feature])).hour
            return X

class TemporalDayofYear(BaseEstimator, TransformerMixin):
        def __init__(self, variable=None, ref_feature=None):
            self.variable = variable
            self.ref_feature = ref_feature

        def fit(self, X, y=None):
            return self

        def transform(self, X):
            X = X.copy()
            X[self.variable] = pd.DatetimeIndex(pd.to_datetime(X[self.ref_feature])).dayofyear
            return X

class SolarElevAngle(BaseEstimator, TransformerMixin):
        def __init__(self, var_name=None, day=None, hour=None):
            self.var_name = var_name
            self.day = day
            self.hour = hour

        def fit(self, X, y=None):
            return self

        def transform(self, X):
            X = X.copy()
            X[self.var_name] = X.apply(lambda x: calc_alpha_s_for_df(N=x[self.day],time=x[self.hour]),axis=1)

            return X

class SunAzimuth(BaseEstimator, TransformerMixin):
        def __init__(self, var_name=None, day=None, hour=None):
            self.var_name = var_name
            self.day = day
            self.hour = hour

        def fit(self, X, y=None):
            return self

        def transform(self, X):
            X = X.copy()
            X[self.var_name] = X.apply(lambda x: calc_sun_azimuth_for_df(N=x[self.day],time=x[self.hour]),axis=1)

            return X

class TheoreticalRadiation(BaseEstimator, TransformerMixin):
        def __init__(self, var_name=None, day=None, hour=None):
            self.var_name = var_name
            self.day = day
            self.hour = hour

        def fit(self, X, y=None):
            return self

        def transform(self, X):
            X = X.copy()
            X[self.var_name] = X.apply(lambda x: calc_glob_irrad(N=x[self.day],time=x[self.hour]),axis=1)

            return X

class DropUnnecessaryFeatures(BaseEstimator, TransformerMixin):
    def __init__(self, variables_to_drop=None):
        self.variables = variables_to_drop

    def fit(self, X, y=None):
        return self

    def transform(self, X):
        X = X.copy()
        X = X.drop(self.variables, axis=1)

        return X



import logging
import sys

# Multiple calls to logging.getLogger('someLogger') return a
# reference to the same logger object.  This is true not only
# within the same module, but also across modules as long as
# it is in the same Python interpreter process.

FORMATTER = logging.Formatter(
    "%(asctime)s - %(name)s - %(levelname)s -" "%(funcName)s:%(lineno)d - %(message)s"
)

def get_console_handler():
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setFormatter(FORMATTER)
    return console_handler



import numpy as np
from sklearn.model_selection import train_test_split

from regression_model import pipeline
from regression_model.processing.data_management import load_dataset, save_pipeline
from regression_model.config import config
from regression_model import __version__ as _version

import logging

_logger = logging.getLogger(__name__)


def run_training() -> None:
    """Train the model"""

    #read training data
    data = load_dataset(filename=config.TRAINING_DATA_FILE)

    #train test split
    X_train, X_test, y_train, y_test = train_test_split(
        data[config.FEATURES], data[config.TARGET], test_size=config.TEST_SIZE, random_state=config.RANDOM_STATE
    )

    pipeline.energy_pipe.fit(X_train[config.FEATURES], y_train)

    _logger.info(f"saving model version: {_version}")
    save_pipeline(pipeline_to_save=pipeline.energy_pipe)

if __name__ == "__main__":
    run_training()



import numpy as np
import pandas as pd
# from http://sgemfinalreport.fi/files/WP611_photovoltaics%20HP%20Hellman.pdf
''' G_on and general'''
longtitude = 24.9693
latitude = 60.1867
lon = longtitude
phi = latitude
delta_GMT = 0
gamma = 0
beta=45

def calc_sun_azimuth_for_df(N, time, lon=longtitude, delta_GMT=3, phi=latitude):
    delta = calc_delta(N)
    omega = calc_omega(N, lon, delta_GMT, time)
    alpha_s = calc_alpha_s(phi, delta, omega)
    gamma_s = calc_gamma_s(alpha_s, phi, delta, omega)
    return gamma_s

def calc_alpha_s_for_df(N, time, lon=longtitude, delta_GMT=3, phi=latitude):
    delta = calc_delta(N)
    omega = calc_omega(N, lon, delta_GMT, time)
    alpha_s = calc_alpha_s(phi, delta, omega)
    return alpha_s

def calc_sol_time(N, lon, delta_GMT, time):
    sol_time_minus_st_time = calc_sol_time_minus_st_time(N, lon, delta_GMT)
    sol_time = time+sol_time_minus_st_time
    return(sol_time)

def calc_omega(N, lon, delta_GMT, time):
    sol_time = calc_sol_time(N, lon, delta_GMT, time)
    omega = 15 * (sol_time-12)
    return omega

def calc_sol_time_minus_st_time(N, lon, delta_GMT): # TEST PASSED!
    E = calc_eot(N)
    sol_time_minus_st_time = (4 * (lon-15*delta_GMT)+E)/60
    return sol_time_minus_st_time

def calc_eot(N):
    B = (N-1) * 360/365
    E = 229.2*(0.000075 + 0.001868*np.cos(B*np.pi/180) - 0.032077*np.sin(B*np.pi/180) - 0.014615*np.cos(2*B*np.pi/180) - 0.04089*np.sin(2*B*np.pi/180))
    return E

def calc_delta(N):
    delta = 23.45 * np.sin(360*(284+N)*np.pi/180/365)
    return delta

def calc_G_on(N):
    G_on = 1367 * (1 + 0.033 * np.cos(360*N/365*np.pi/180))
    return G_on

''' Direct below'''
def calc_alpha_s(phi, delta, omega):
    #sind(phi)*sind(delta(x))+cosd(phi)*cosd(delta(x))*cosd(h(x,k))
    #alfa_s(x,k)=asind(sind(phi)*sind(delta(x))+cosd(phi)*cosd(delta(x))*cosd(h(x,k)))
    part1 = np.sin(phi*np.pi/180) * np.sin(delta*np.pi/180)
    part2 = np.cos(phi*np.pi/180) * np.cos(delta*np.pi/180) * np.cos(omega*np.pi/180)
    in_arcsin = part1 + part2
    alpha_s = (np.arcsin(in_arcsin)*180)/np.pi #converting rad->deg
    return alpha_s

def calc_gamma_s(alpha_s, phi, delta, omega):
    top = np.sin(alpha_s*np.pi/180) * np.sin(phi*np.pi/180) - np.sin(delta*np.pi/180)
    bot = np.cos(alpha_s*np.pi/180) * np.cos(delta*np.pi/180)
    in_arccos = top / bot # this is in rad
    in_abs = np.arccos(in_arccos) * 180/np.pi #this is in degrees
    gamma_s = np.sign(omega) * np.absolute(in_abs)
    return gamma_s

def calc_cos_theta_i(alpha_s, gamma_s, beta=45, gamma=0):
    part1 = np.sin(alpha_s*np.pi/180) * np.cos(beta*np.pi/180)
    d_gamma = gamma_s-gamma
    part2 = np.cos(alpha_s*np.pi/180) * np.sin(beta*np.pi/180) * np.cos(d_gamma*np.pi/180)
    cos_theta_i =  part1 + part2 #this is in radian
    return cos_theta_i

def calc_m(alpha_s, z=-53):
    z_h = 8434.5
    pp0 = np.exp(-z/z_h)
    alpha_s_true = calc_alpha_s_true(alpha_s)
    bot = np.sin(alpha_s_true*np.pi/180) + 0.50572 * ((alpha_s_true + 6.07995)**(-1.6364))
    m = pp0/bot
    return m

def calc_alpha_s_true(alpha_s):
    d_alpha_ref = calc_d_alpha_ref(alpha_s)
    alpha_s_true = alpha_s + d_alpha_ref
    return alpha_s_true

def calc_d_alpha_ref(alpha_s):
    part1 = 0.061359 * (180/np.pi)
    top2 = 0.1594 + 1.123*(np.pi/180)*alpha_s + 0.065656*((np.pi/180)**2)*(alpha_s**2)
    bot3  = 1 + 28.9344*(np.pi/180)*alpha_s + 277.3971*((np.pi/180)**2)*(alpha_s**2)
    d_alpha_ref = part1 * (top2 / bot3)
    return d_alpha_ref

def calc_d_R(m):
    if m<=20:
        bot = 6.6296 + 1.7514*m - 0.1202*(m**2) + 0.0065*(m**3) - 0.00013*(m**4)
    else:
        bot = 10.4 + 0.718*m
    d_R = 1 / bot
    return d_R

def calc_direct_irrad(G_on, alpha_s, cos_theta_i, T_L=2):
    m = calc_m(alpha_s)
    d_R = calc_d_R(m)
    in_exp = -0.8662 * T_L * m * d_R
    G_beam = G_on * cos_theta_i * np.exp(in_exp)
    return G_beam

''' diffuse below'''

def calc_t_rd(T_L=2):
    T_rd = -0.015843 + 0.030543*T_L + 0.0003797*(T_L**2)
    return T_rd

def calc_f_d(T_rd, alpha_s, T_L=2):
    A0 = 0.26463 - 0.061581*T_L + 0.0031408*(T_L**2)
    if (A0*T_rd <0.002):
        A0 = 0.002/T_rd
    A1 = 2.0402 + 0.018945*T_L - 0.011161*(T_L**2)
    A2 = 1.3025 + 0.039231*T_L + 0.0085079*(T_L**2)
    F_d = A0 + A1*np.sin(alpha_s*np.pi/180) + A2*(np.sin(alpha_s*np.pi/180)**2)
    return F_d

def calc_diffuse_irrad(T_rd, alpha_s, G_on, beta):
    F_d = calc_f_d(T_rd, alpha_s, T_L=2)
    G_d = G_on * T_rd * F_d * ((1+np.cos(beta*np.pi/180))/2)
    return G_d


def calc_glob_irrad(N,time,lon=longtitude,delta_GMT=0,phi=latitude):
    gamma = 0
    beta=45
    omega = calc_omega(N, lon, delta_GMT, time)
    delta = calc_delta(N)
    alpha_s = calc_alpha_s(phi, delta, omega)
    if alpha_s>0:
        gamma_s = calc_gamma_s(alpha_s, phi, delta, omega)
        cos_theta_i = calc_cos_theta_i(alpha_s, gamma_s, gamma, beta)
        if cos_theta_i>0:
            G_on = calc_G_on(N)
            G_beam = calc_direct_irrad(G_on, alpha_s, cos_theta_i, T_L=2)
            T_rd = calc_t_rd(T_L=2)
            G_d = calc_diffuse_irrad(T_rd, alpha_s, G_on, beta)
            G_glob = G_beam + G_d
        else:
            G_glob = 0
    else:
        G_glob = 0
    return G_glob



import pandas as pd
import joblib
from sklearn.pipeline import Pipeline

from regression_model.config import config
from regression_model import __version__ as _version

import logging

_logger = logging.getLogger(__name__)

def load_dataset(*, filename: str) -> pd.DataFrame:
    _data = pd.read_csv(f"{config.DATASET_DIR}/{filename}")
    _data = _data.set_index(pd.DatetimeIndex(pd.to_datetime(_data[config.DATETIME_INDEX], utc=True),dayfirst=True))
    return _data

def save_pipeline(*, pipeline_to_save) -> None:
    """
    Save the pipeline.
    Saves the versioned model, and overwrites any previous
    saved models. This ensures that when the package is
    published, there is only one trained model that can be
    called, and we know exactly how it was built.
    """
    # Prepare versioned save file name
    save_file_name = f"{config.PIPELINE_SAVE_FILE}{_version}.pkl"
    save_path = config.TRAINED_MODEL_DIR / save_file_name

    remove_old_pipelines(files_to_keep=save_file_name)
    joblib.dump(pipeline_to_save, save_path, compress=3)
    _logger.info(f"saved pipeline: {save_file_name}")

def load_pipeline(*, filename: str) -> Pipeline:
    """Load saved pipeline"""
    file_path = config.TRAINED_MODEL_DIR / filename
    trained_model = joblib.load(filename = file_path)
    return trained_model

def remove_old_pipelines(*, files_to_keep) -> None:
    """
    Remove old model pipelines.
    This is to ensure there is a simple one-to-one
    mapping between the package version and the model
    version to be imported and used by other applications.
    """
    for model_file in config.TRAINED_MODEL_DIR.iterdir():
        if model_file.name not in [files_to_keep, "__init__.py"]:
            model_file.unlink()



from sklearn.pipeline import Pipeline
from sklearn.ensemble import RandomForestRegressor
from sklearn.preprocessing import StandardScaler
# from sklearn.preprocessing import MinMaxScaler #select  one of the two

from regression_model.processing import preprocessors as pp
from regression_model.config import config
# from regression_model.processing import features

import logging

_logger = logging.getLogger(__name__)

energy_pipe = Pipeline(
    [
        ('wind_disc', pp.WindDiscretizer(variables=config.WIND_DISCRETE)),
        ('num_to_binary', pp.DiscretizerNumericalIntoBinary(boundaries=config.BINARY_BOUNDARIES, variables=config.NUM_TO_BINARY)),
        ('temporal_hour', pp.TemporalHour(variable=config.TEMPORAL_HOUR, ref_feature=config.DATETIME_INDEX)),
        ('temporal_dayofyear', pp.TemporalDayofYear(variable=config.TEMPORAL_DAY, ref_feature=config.DATETIME_INDEX)),
        ('solar_angle', pp.SolarElevAngle(var_name=config.SOLAR_ANGLE, day=config.TEMPORAL_DAY, hour=config.TEMPORAL_HOUR)),
        ('sun_azimuth', pp.SunAzimuth(var_name=config.SUN_AZIMUTH, day=config.TEMPORAL_DAY, hour=config.TEMPORAL_HOUR)),
        ('theor_solar_radiation', pp.TheoreticalRadiation(var_name=config.THEOR_SRAD, day=config.TEMPORAL_DAY, hour=config.TEMPORAL_HOUR)),
        ('drop_features', pp.DropUnnecessaryFeatures(variables_to_drop=config.DROP_FEATURES)),
        ("scaler", StandardScaler()),
        ("random_forest", RandomForestRegressor(n_estimators=300, min_samples_split=5, min_samples_leaf=1, max_features='sqrt',max_depth=30, bootstrap=False, n_jobs= -1, random_state=config.RANDOM_STATE))
    ]
)



from regression_model.config import config
import pandas as pd

def validate_inputs(input_data: pd.DataFrame) -> pd.DataFrame:
    # check for numerical variables with NA not seen during training
    validated_data = input_data.copy()
    if input_data[config.FEATURES].isnull().any().any():
        validated_data = validated_data.dropna(
            axis=0, subset=config.FEATURES)

    return validated_data



import logging

from regression_model.config import config
from regression_model.config import logging_config

VERSION_PATH = config.PACKAGE_ROOT / 'VERSION'

#configure the logger for use in  the package
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(logging_config.get_console_handler())
logger.propagate = False

with open(VERSION_PATH, 'r') as file_version:
    __version__ = file_version.read().strip()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
from pathlib import Path
from setuptools import find_packages, setup

#package meta-data.
NAME = 'solar_prod_suvilahti_ml_model'
DESCRIPTION = 'Study project. A regression model to predict Solar Energy Production.'
URL = 'https://github.com/screwdriver66/solar_prod_suvilahti'
EMAIL = 'simo.uimonen@gmail.com'
AUTHOR = 'Semen Uimonen'
REQUIRES_PYTHON = '>=3.6.0'

#package requirements
def list_reqs(filename='requirements.txt'):
    with open(filename) as f:
        return f.read().splitlines()

# CHECK LICENSE AND TROVE CLASSIFIERS!!!!
here = os.path.abspath(os.path.dirname(__file__))
#As a README for a long discription:
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# load the package __version__.py module as a dictionary
ROOT_DIR = Path(__file__).resolve().parent
PACKAGE_DIR = ROOT_DIR / 'regression_model'
about = {}
with open(PACKAGE_DIR / 'VERSION') as f:
    _version = f.read().strip()
    about['__version__'] = _version

VERSION = _version

setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=('tests',)),
    package_data={'regression_model': ['VERSION']},
    install_requires=list_reqs(),
    extras_require={},
    include_package_data=True,
    license='GPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],

)



from fmi import FMI
from regression_model.config import config
from datetime import date, datetime
import pandas as pd
import numpy as np

def get_forecast(place):
    f = FMI(place=place)
    forecast = f.forecast()
    #make it into a dataframe
    df = pd.DataFrame()
    for i,observation in enumerate(forecast):
        df_temp = pd.DataFrame(observation.__dict__, index=np.arange(i,i+1))
        df = pd.concat([df,df_temp], axis=0)

    #get the global radiation to proper values
    df[config.GLOBAL_RADIATION] = df[config.RADIATION_ACCUMULATION].diff()/3600
    # discard first row
    df = df.loc[1:]
    df =  df.drop(columns=config.FORECAST_TO_DROP)
    df.columns = config.FEATURES
    # to go along with WeatherDataRequestSchema of the API we change
    # 'datetime_converted' to a string
    df[config.DATETIME_INDEX] = df[config.DATETIME_INDEX].astype(str)
    return df



import numpy as np
import pandas as pd

from regression_model.processing.data_management import load_pipeline
from regression_model.config import config
from regression_model.processing.validation import validate_inputs
from regression_model import __version__ as _version
import logging

_logger = logging.getLogger(__name__)

pipeline_file_name = f"{config.PIPELINE_SAVE_FILE}{_version}.pkl"
_energy_pipe = load_pipeline(filename = pipeline_file_name)

def make_prediction(*, input_data) -> dict:
    """Make predictions using the saved model pipeline."""

    data = pd.DataFrame(input_data)
    validated_data = validate_inputs(input_data=data)
    prediction = _energy_pipe.predict(validated_data[config.FEATURES])

    results = {"predictions": prediction, "version": _version}

    _logger.info(
     f"Making predictions with model version: {_version} "
     f"Inputs: {validated_data} "
     f"Predictions: {results}"
    )

    return results
